/*
 * Copyright 2015 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

Item {
    property int currentIndex: -1
    property var currentTab: null
    property bool behaveAsStack: false
    property alias count: tabs.count

    // Expose the ListModel through a property instead of inheriting directly
    // from it due to a bug in QML that does not allow calling base class
    // methods: https://bugreports.qt.io/browse/QTBUG-26357
    // This allows us to create our own methods that implement the required
    // semantics (such as correctly tracking and updating currentIndex and
    // currentTab) at the price of the minor inconvenience of having to access
    // TabsModel.listModel when wanting to use it as model for a view.
    property ListModel listModel: ListModel {
        id: tabs
    }

    onCurrentIndexChanged: internal.updateCurrentTab()
    onBehaveAsStackChanged: if (behaveAsStack) internal.updateCurrentTab()

    function validIndex(index) { return index >= 0 && index < tabs.count }

    function add(tab) {
        if (tab) {
            var index = tabs.count
            tabs.append({tab: tab})
            if (index === 0) {
                // the first item added to an empty model always becomes
                // the new current item
                currentIndex = 0
            }
            else if (index == currentIndex) {
                // the current index was set out of bounds above the maximum
                // and by adding an item we made it valid, so we need to update
                // the current tab to match
                internal.updateCurrentTab()
            }
            return index
        } else {
            console.log("TabsModel.add: Invalid Tab", tab)
            return -1;
        }
    }

    function remove(index) {
        if (!validIndex(index)) return null
        var tab = tabs.get(index).tab
        tabs.remove(index)

        // if we removed the current index and it is the last item OR
        // if we removed any item before the current index,
        // then decrement the current index
        if (index < currentIndex) {
            --currentIndex
        } else if (index === currentIndex) {
            if (index === tabs.count) --currentIndex
            else internal.updateCurrentTab() // next item becomes current
        }

        return tab;
    }

    function get(index) {
        return validIndex(index) ? tabs.get(index).tab : null
    }

    function move(from, to)
    {
        if (from == to || !validIndex(from) || !validIndex(to)) {
            return
        }
        tabs.move(from, to, 1)
        if (currentIndex == from) {
            currentIndex = to
        } else if (currentIndex >= to && currentIndex < from) {
            currentIndex++
        } else if (currentIndex > from && currentIndex <= to) {
            currentIndex--
        }
    }

    QtObject {
        id: internal
        function updateCurrentTab() {
            var newCurrentTab = null
            if (validIndex(currentIndex)) {
                if (behaveAsStack) {
                    tabs.move(currentIndex, 0, 1)
                    currentIndex = 0
                }
                newCurrentTab = tabs.get(currentIndex).tab
            }
            if (newCurrentTab != currentTab) {
                currentTab = newCurrentTab
            }

        }
    }
}
