/*
 * Copyright 2015 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import QtTest 1.0
import "../../../src/app/webbrowser"

Item {
    width: 600
    height: 50

    property alias model: modelLoader.item

    Loader {
        id: modelLoader
        active: false
        asynchronous: false
        sourceComponent: TabsModel { }
    }

    Component {
        id: tab
        Item {
            property url url
            property string title
            property url icon
        }
    }

    SignalSpy {
        id: countSpy
        target: model
        signalName: "countChanged"
    }

    SignalSpy {
        id: currentIndexSpy
        target: model
        signalName: "currentIndexChanged"
    }

    SignalSpy {
        id: currentTabSpy
        target: model
        signalName: "currentTabChanged"
    }

    TestCase {
        name: "TabsModel"
        when: windowShown

        function createTab()
        {
            return tab.createObject()
        }

        function init()
        {
            modelLoader.active = true
        }

        function cleanup()
        {
            modelLoader.active = false
            countSpy.clear()
            currentIndexSpy.clear()
            currentTabSpy.clear()
        }

        function test_shouldBeInitiallyEmpty()
        {
            compare(model.count, 0)
            compare(model.currentIndex, -1)
            compare(model.currentTab, null)
        }

        function test_currentTabShouldBeNullOnInvalidIndex_data()
        {
            return [{index: 0}, {index: 2}, {index:-2}]
        }

        function test_currentTabShouldBeNullOnInvalidIndex(data)
        {
            model.currentIndex = data.index
            compare(model.currentIndex, data.index)
            compare(model.currentTab, null)
        }

        function test_currentTabShouldBeValidWhenIndexBecomesValid()
        {
            // preload one item into the model before setting the index because
            // the first item added will set the current index to zero.
            model.add(createTab())
            model.currentIndex = 2
            compare(model.count, 1)
            compare(model.currentIndex, 2)
            compare(model.currentTab, null)
            currentTabSpy.clear()

            // now we can test that until we add enough tabs to reach the
            // current index, the currentTab will remain null
            model.add(createTab())
            var tab = createTab()
            model.add(tab)

            compare(model.count, 3)
            compare(model.currentIndex, 2)
            compare(model.currentTab, tab)
            compare(currentTabSpy.count, 1)
        }

        function test_shouldNotAddNullTab()
        {
            compare(model.add(null), -1)
            compare(model.count, 0)
        }

        function test_shouldReturnIndexWhenAddingTab()
        {
            for (var i = 0; i < 3; ++i) {
                compare(model.add(createTab()), i)
            }
        }

        function test_shouldUpdateCountWhenAddingTab()
        {
            model.add(createTab())
            compare(countSpy.count, 1)
            compare(model.count, 1)
        }

        function test_shouldUpdateCountWhenRemovingTab()
        {
            model.add(createTab())
            countSpy.clear()
            delete model.remove(0)
            compare(countSpy.count, 1)
            compare(model.count, 0)
        }

        function test_shouldNotAllowRemovingAtInvalidIndex()
        {
            compare(model.remove(0), null)
            compare(model.remove(2), null)
            compare(model.remove(-2), null)
        }

        function test_shouldReturnTabWhenRemoving()
        {
            var tab = createTab()
            model.add(tab)
            var removed = model.remove(0)
            compare(removed, tab)
            delete removed;
        }

        function test_shouldNotChangeCurrentTabWhenAddingUnlessModelWasEmpty()
        {
            var tab = createTab()
            model.add(tab)
            compare(currentTabSpy.count, 1)
            compare(model.currentTab, tab)
            currentTabSpy.clear()
            model.add(createTab())
            compare(currentTabSpy.count, 0)
            compare(model.currentTab, tab)
        }

        function test_shouldUpdateCurrentTabWhenSettingCurrentIndex()
        {
            var tab1 = createTab()
            model.add(tab1)
            compare(model.currentIndex, 0)
            compare(currentTabSpy.count, 1)
            compare(model.currentTab, tab1)

            var tab2 = createTab()
            model.add(tab2)
            model.currentIndex = 1
            compare(model.currentIndex, 1)
            compare(currentTabSpy.count, 2)
            compare(model.currentTab, tab2)
        }

        function test_shouldUpdateCurrentTabWhenRemoving()
        {
            // Adding a tab to an empty model should update the current tab.
            // Removing the last tab from the model should update it too.
            model.add(createTab())
            currentTabSpy.clear()
            currentIndexSpy.clear()
            delete model.remove(0)
            compare(currentTabSpy.count, 1)
            compare(currentIndexSpy.count, 1)

            // When removing a tab after the current one, neither the
            // current tab nor the current index should change.
            var tab1 = createTab()
            model.add(tab1)
            model.add(createTab())
            currentTabSpy.clear()
            currentIndexSpy.clear()
            delete model.remove(1)
            compare(model.currentTab, tab1)
            compare(currentTabSpy.count, 0)
            compare(currentIndexSpy.count, 0)

            // When removing the current tab, if there is a tab after it, it
            // becomes the current one, and thus the current index doesn’t change.
            var tab2 = createTab()
            model.add(tab2)
            currentTabSpy.clear()
            currentIndexSpy.clear()
            delete model.remove(0)
            compare(currentTabSpy.count, 1)
            compare(currentIndexSpy.count, 0)
            compare(model.currentTab, tab2)

            // When removing a tab before the current one, the current
            // tab doesn’t change but the current index is updated.
            var tab3 = createTab()
            model.add(tab3)
            model.currentIndex = 1
            currentTabSpy.clear()
            currentIndexSpy.clear()
            delete model.remove(0)
            compare(currentTabSpy.count, 0)
            compare(currentIndexSpy.count, 1)
            compare(model.currentIndex, 0)

            // When removing the current tab, if it was the last one, the
            // current tab should be reset to 0.
            currentTabSpy.clear()
            currentIndexSpy.clear()
            delete model.remove(0)
            compare(currentTabSpy.count, 1)
            compare(currentIndexSpy.count, 1)
            compare(model.currentTab, null)
            compare(model.currentIndex, -1)
        }

        function test_shouldReturnTabAtIndex()
        {
            // valid indexes
            var tab1 = createTab()
            model.add(tab1)
            var tab2 = createTab()
            model.add(tab2)
            var tab3 = createTab()
            model.add(tab3)
            compare(model.get(0), tab1)
            compare(model.get(1), tab2)
            compare(model.get(2), tab3)

            // invalid indexes
            compare(model.get(-1), null)
            compare(model.get(3), null)
        }

        function test_shouldPreserveCurrentIndexAndTabWhenMoving_data()
        {
            return [
                {initial: 0, from: 0, to: 2, result: 2}, // moving tab that has the current index
                {initial: 1, from: 0, to: 2, result: 0}, // moving from before current index to after current index
                {initial: 1, from: 2, to: 0, result: 2}  // moving from after current index to before current index
            ]
        }
        function test_shouldPreserveCurrentIndexAndTabWhenMoving(data)
        {
            for (var i = 0; i < 3; ++i) {
                model.add(createTab())
            }
            model.currentIndex = data.initial
            var currentTabInitial = model.currentTab
            model.move(data.from, data.to)
            compare(model.currentIndex, data.result)
            compare(model.currentTab, currentTabInitial)
        }

        function test_shouldMoveToTopWhenSettingCurrentIndexInStackMode() {
            var tab1 = createTab()
            model.add(tab1)
            var tab2 = createTab()
            model.add(tab2)
            var tab3 = createTab()
            model.add(tab3)
            model.behaveAsStack = true
            compare(model.currentIndex, 0)

            model.currentIndex = 2
            compare(model.currentIndex, 0)
            compare(model.currentTab, tab3)
            compare(model.get(1), tab1)
            compare(model.get(2), tab2)
        }
    }
}
